//----- Include files ---------------------------------------------------------
#include <algorithm>
#include <future>
#include <iostream>
#include <vector>
#include "network.hpp"

//----- HTTP response messages ----------------------------------------------
#define OK_IMAGE    "HTTP/1.0 200 OK\nContent-Type:image/gif\n\n"
#define OK_TEXT     "HTTP/1.0 200 OK\nContent-Type:text/html\n\n"
#define NOTOK_404   "HTTP/1.0 404 Not Found\nContent-Type:text/html\n\n"
#define MESS_404    "<html><body><h1>FILE NOT FOUND</h1></body></html>"

int main(void)
{
    std::vector<std::future<void>> pending_futures;
    Server proxy(kProxyPort);
    Client browser;
    Client webserver;

	// Main loop to listen, accept, and then spin-off a thread to handle the GET
	while (1)
	{
        if (browser.ConnectFrom(proxy.socketFD) != 0)
        {
            std::cerr << "ERROR - Unable to create socket to client" << std::endl;
            continue;
        }
        if (webserver.ConnectTo(kWebserverPort) != 0)
        {
            std::cerr << "ERROR - Unable to connect to webserver" << std::endl;
            continue;
        }
        auto newThreadRequest1 = std::async(std::launch::async, PipeSockets, browser.socketFD, webserver.socketFD);
        auto newThreadRequest2 = std::async(std::launch::async, ProxySockets, webserver.socketFD, browser.socketFD);
        pending_futures.push_back(std::move(newThreadRequest1));
        pending_futures.push_back(std::move(newThreadRequest2));
        std::cout << "Returned from creating threads, continuing..." << std::endl;
	}
    proxy.Close();
	return 0;
}

